local blocks = require("blocks.blocks")
local fileutils = require("utils.fileutils")
local Point = require("point").Point
local Chunk = {}
Chunk.totalSaving = 0
Chunk.totalLoading = 0
Chunk.saveCount = 0
Chunk.loadCount = 0
function Chunk.getStats()
  local result = ""
  
  if Chunk.saveCount > 0 then
    result = result .. "saves=".. tostring(Chunk.totalSaving / Chunk.saveCount)
  else
    result = result .. "no saves"
  end
  result = result.."\t\t\t"
  if Chunk.loadCount > 0 then
    result = result .. "loads="..tostring(Chunk.totalLoading / Chunk.loadCount)
  else
    result = result .. "no loads"
  end
  return result
end
function Chunk:new(layer, fromPoint, toPoint)
  self.__index = self
  local instance = {layer=layer, fromPoint=fromPoint,toPoint=toPoint, isLoaded=false}
  setmetatable(instance,self)
  return instance
end

function Chunk:load()
  local start = love.timer.getTime()
  if self.isLoaded then
    return
  end
  if self:hasSave() then
    self:loadFromDisk()
  else
    self:generate()
  end
  self.isLoaded = true
  
  Chunk.loadCount = Chunk.loadCount + 1
  Chunk.totalLoading = Chunk.totalLoading + love.timer.getTime() - start
end

function Chunk:unload()
  local start = love.timer.getTime()
  if not self.isLoaded then
    return
  end
  --print("UNloading chunk "..tostring(self.fromPoint).."-"..tostring(self.toPoint))
  io.flush()
  self:save()
  for x = self.fromPoint.x,self.toPoint.x do
    for y = self.fromPoint.y,self.toPoint.y do
      self.layer:clearBlock(x,y)
    end
  end
  
  self.isLoaded = false
  
  Chunk.saveCount = Chunk.saveCount + 1
  Chunk.totalSaving = Chunk.totalSaving + love.timer.getTime() - start
end

function Chunk:getId()
  return "chunk"..tostring(self.fromPoint.x).."_"..tostring(self.fromPoint.y).."_"..tostring(self.toPoint.x).."_"..tostring(self.toPoint.y).."("..tostring(self.layer.index)..")"
end

function Chunk:getFileName()
  return fileutils.getChunkFileNameById(self:getId())
end

function Chunk:save()
  
  local fromPoint = self.fromPoint
  local toPoint = self.toPoint
  local lines = {}
  for x=fromPoint.x, toPoint.x do
    for y=fromPoint.y, toPoint.y do
      local block = self.layer:getBlock(x,y)
      
      lines[#lines+1] = tostring(x).." "..tostring(y).." "..block.blockId.." "..block:getSaveString()      
    end
  end
  local content = table.concat(lines,"\n")
  
  local fileName = self:getFileName()
  --local file, _error = love.filesystem.newFile(fileName, "w")  
  --file:setBuffer("full",100000)
  --assert(file, "file object wasn't created! Filename="..tostring(fileName).." error="..tostring(_error))
  --love.filesystem.write(fileName, table.concat(lines))
  local handle, _error = io.open(love.filesystem.getSaveDirectory().."/".. fileName, "w")
  if _error then
    print("file error " .._error)
  end
  handle:write(content)
  handle:close()
end

function Chunk:generate()
  --print("loading chunk "..tostring(self.fromPoint)..";"..tostring(self.toPoint))
  io.flush()
  for x=self.fromPoint.x, self.toPoint.x do
    for y = self.fromPoint.y, self.toPoint.y do
      self.layer:spawnBlock(x,y, self.layer.generator:getBlockXY(x,y, self.layer.index))
    end
  end
end

function Chunk:hasSave()
  return love.filesystem.exists(self:getFileName())
end



function Chunk:loadFromDisk()
  local fileName = self:getFileName()
  
  local data
  local _error
  local path = love.filesystem.getSaveDirectory().."/"..fileName
  local file = io.open(path, "r")
  data = file:read("*a")
  file:close()    

    
  local fromPoint = self.fromPoint
  local toPoint = self.toPoint  
  
  for line in data:gmatch("[^\n]+") do 
    assert(line~="")
    local nextToken = line:gmatch("[^ ]+")
    local x = tonumber(nextToken())
    local y = tonumber(nextToken())
    local blockId = nextToken()    
    
    local remaining = {}
    for part in nextToken do
      remaining[#remaining + 1] = part
    end
    remaining = table.concat(remaining)
    
    local block = blocks.new(blockId)
    block:fromSaveString(remaining)
    assert(block)
    self.layer:spawnBlock(x,y, block)
  end
  
end


return Chunk